#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
//#include "xuart.h"

/*

(c)2009 Technologic Systems
Author: Michael Schmidt

This program was created as an example program in the hopes of being useful 
to customers trying to control DMX lighting from Technologic Systems products.  
It is free to use and has no warranty.

This program shows a sample of how to control DMX lighting.  It was designed
to work with 3 lights:
LIGHT1 starts at address 1
LIGHT2 starts at address 5
LIGHT3 starts at address 9

The DMX channel list is:
Channel 1: Dimmer/Strobe
Channel 2: Red
Channel 3: Green
Channel 4: Blue

Before running this program, start the xuartctl program to talk to the xuarts.

By way of example, suppose you want to run this program on your PC, your
TS-73XX is at 192.168.0.59, and your DMX lights are connect to XUART #3.
First, run this command on the TS-73XX:

xuartctl --server

Then run this command on your PC:

xuartctl --port=192.168.0.59:7353 --server --speed=250000 --mode=dmx,raw

Now you are ready to run this program, which will do the actual work of
creating packets of raw XUART opcodes and sending them to control the DMX 
lighting.  It is necessary to use raw XUART opcodes (specified in the mode
string above by "raw") because DMX packets require break and idle times of
specific minimum lengths (88us + 8us) to denote that a packet is about to
start.  We use 400us for the break and 40us for the idle.

Notes:
1. The xuartctl version for the PC will of course be different from the
same named program on the TS-73XX, due to different CPU architectures.
2. If xuartctl is not in your path you will need to specify that.

 */

static inline unsigned short ch_break(int bits) {
  unsigned short opcode;

  // assert(bits < 1024*1024)
  if (bits > 1023) {
    opcode = 0x0800 + bits / 1024;
  } else {
    opcode = 0x0800 + bits;
  }
  return opcode;
}

static inline unsigned short ch_idle(int bits) {
  unsigned short opcode;

  // assert(bits < 1024*1024)
  if (bits > 1023) {
    opcode = 0x1000 + bits / 1024;
  } else {
    opcode = 0x1000 + bits;
  }
  return opcode;
}

static inline unsigned short ch_data(unsigned char data) {
  unsigned short opcode = 0x100 + data;

  return opcode;
}

// dmxtest2 <red> <green> <blue> <dimmer/strobe>
int main(int argc, char **argv) {
  int run,i,fd,r=0,g=0,b=0,ds=255,dr=1,dg=1,db=1,dds=-1;
  char buf[513];
  unsigned short txbuf[515];

  if (argc < 2) {
    fprintf(stderr,"need to specify device\n");
    return 1;
  }
  fd = open(argv[1],O_RDWR);
  if (fd < 0) {
    perror(argv[1]);
    return 1;
  }
  if (argv[2]) {
    r = atoi(argv[2]);
    if (argv[3]) {
      g = atoi(argv[3]);
      if (argv[4]) {
	b = atoi(argv[4]);
	if (argv[5]) {
	  ds = atoi(argv[5]);
	}
      }
    }
  }
  for (;;) {
    txbuf[0] = ch_break(100);
    txbuf[1] = ch_idle(10);
    txbuf[2] = ch_data(0);

    txbuf[3] = ch_data(ds);
    txbuf[4] = ch_data(0);
    txbuf[5] = ch_data(0);
    txbuf[6] = ch_data(r);

    txbuf[7] = ch_data(ds);
    txbuf[8] = ch_data(0);
    txbuf[9] = ch_data(g);
    txbuf[10] = ch_data(0);

    txbuf[11] = ch_data(ds);
    txbuf[12] = ch_data(b);
    txbuf[13] = ch_data(0);
    txbuf[14] = ch_data(0);

    for (i=15;i<515;i++) txbuf[i] = ch_data(0);
    /*
    if (n > 0 && n < 512) {
      txbuf[3+n] = ch_data(run);
    }
    */
    write(fd,txbuf,515*sizeof(unsigned short));
    
    r += dr;
    if (dr && (r >= 152 || r <= 0)) {
      dr = -dr;
    }
    g += dg;
    if (dg && (g >= 152 || g <= 0)) {
      dg = -dg;
    }
    b += db;
    if (db && (b >= 152 || b <= 0)) {
      db = -db;
    }
    //break;
    fprintf(stderr,"%d  %d+%d %d+%d %d+%d     \r",ds,r,dr,g,dg,b,db);
    usleep(10000);
  }
}
